const reactNativeInstall = module.exports
const PUB_SUB = require('../../utils/pub-sub')
const LOG = require('../../utils/logger')
const FILE_HELPER = require('../../utils/file-helper')
const xcode = require('xcode')
const IOS = require('../../platforms/ios')
const CONSTANTS = require('../../constants')
const {XCODE_BUILD_PHASES, IMPORT_LINES, PATHS} = CONSTANTS
const REACT_NATIVE_HELPERS = require('./react-native-helpers')
const path = require('path');

/**
 * Add all the New Relic specific statements to a React Native app
 * @param args
 */
reactNativeInstall.handleReact = function (args = {}) {
  LOG.info('Setting up New Relic for React Native')

  if (!args.platforms || !args.platforms.length) {
    LOG.error("Cannot determine the React Native platform. Please use --platform option")
    return
  }

  const newRelicFilePath = args.newRelicConfig || CONSTANTS.newRelicConfig
  const rnAppFilePath = args.rnAppFilePath || PATHS.rnApp
  const platforms = args.platforms

  FILE_HELPER
    .checkForFile(newRelicFilePath)
    .then((newRelicConfig) => {
      //TODO: This is case sensitive. Need to allow for app.js and App.js
      REACT_NATIVE_HELPERS.injectNewRelicImport(rnAppFilePath)
        .then((err) => {
          if (!err) {
            LOG.info(`Added New Relic import successfully`)
          }
        })
        .catch((err) => {
          LOG.debug(err)
        });

      platforms.map((platform) => {
        PUB_SUB.publish(`rn.${platform.toLowerCase()}`, args)
      })
    })
    .catch(() => {
      LOG.error("A newrelic.json file is required to link New Relic. Please include this config file in the root of your app.")
    })
}

/**
 * Adds iOS-specific changes to a React Native app to use the New Relic agent.
 * Will fail if no xcode project file or newrelic.json file found.
 * @param args
 */
reactNativeInstall.handleIos = function (args = {}) {
  const xCodeFilePath = args.xCodeFilePath || PATHS.xcodePbxproj
  FILE_HELPER
    .checkForFile(xCodeFilePath)
    .then((files) => {
      //hardcoded to the first file for now...
      const xcodeProjectFile = files[0]
      const xcodeProject = xcode.project(xcodeProjectFile);
      xcodeProject.parse((error) => {
        if (error) {
          LOG.debug(`Xcode loading error: ${error.code}`);
          LOG.error('Encountered an error loading the Xcode project file');
          return error
        }

        LOG.debug(`Found xcode project file: ${xcodeProjectFile}`)
        const iOSProject = new IOS(xcodeProject)
        FILE_HELPER
          .checkForFile(CONSTANTS.newRelicConfig)
          .then((newRelicConfig) => {
            if(!newRelicConfig || newRelicConfig.length == 0) {
              LOG.error('Failed to find newrelic.json')
              return
            }

            const configJsonPath = path.join(process.cwd(), CONSTANTS.newRelicConfig)
            let configJson = ''
            try {
              configJson = require(configJsonPath)
            } catch(e) {
              LOG.error(`Unable to find the newrelic.json file at ${configJsonPath}: ${e}`)
              process.exit(1)
            }
            
            const uploadScriptBash = `export REACT_ROOTDIR=..\nexport PLATFORM=ios\n \nexport buildType=\"\$\{CONFIGURATION\}\"\n bash ../${PATHS.genSourceMapScript}`
            const buildScriptsCopy = iOSProject.getBuildScripts('PBXShellScriptBuildPhase')
            iOSProject
              .prepare(buildScriptsCopy)
              .addShellScriptBuildPhase(XCODE_BUILD_PHASES.uploadMaps, uploadScriptBash)
              .addShellScriptBuildPhase(XCODE_BUILD_PHASES.postBuild, `bash ../${PATHS.iosPostBuildScript} ${configJson.licenseKey}`)
              .finish()
            
            const resourcesBuildPhase = iOSProject.getBuildScripts('PBXResourcesBuildPhase')
            iOSProject
              .prepare(resourcesBuildPhase)
              .addCopyNewRelicConfigBuildPhase(XCODE_BUILD_PHASES.copyNewRelicJson, `../${CONSTANTS.newRelicConfig}`)
              .finish()

            const modifiedProject = iOSProject.getXcodeProject().writeSync();
            FILE_HELPER.writeFile(xcodeProjectFile, modifiedProject)
              .then((err) => {
                if (err) {
                  LOG.debug(`Failed to update: ${xcodeProjectFile}`)
                }
                LOG.info(`Updated Xcode successfully`)
              })
          })
          .catch((e) => {
            LOG.debug(`${e}`)
          })
      })
    })
}

/**
 * Adds Android-specific changes to a React Native app to use the New Relic agent.
 * @param args
 */
reactNativeInstall.handleAndroid = function (args = {}) {
  const androidGradlePath = args.androidAppBuild || PATHS.appBuildGradle
  FILE_HELPER
    .checkForFile(androidGradlePath)
    .then((files) => {
      if (files && files.length) {
        files.map((file) => {
          FILE_HELPER
            .readFileSync(file)
            .then((fileBlob) => {
              LOG.info('Adding New Relic Gradle plugin')
              if (fileBlob.match(IMPORT_LINES.applyNewRelicGradle)) {
                LOG.debug(`New Relic Gradle plugin already exists`)
                return
              }
              // Searching for the RN plugin based on the RN Android template
              // https://github.com/facebook/react-native/blob/2fd50882b56aa16befd5cfbc3c6b27e53c72474e/template/android/app/build.gradle#L84
              const reactGradlePlugin = `apply from: "../../node_modules/react-native/react.gradle"`
              if (!fileBlob.match(reactGradlePlugin)) {
                LOG.debug(`React Native Gradle plugin not installed`)
                return
              }

              fileBlob = fileBlob.replace(new RegExp(`[\n\r] *${reactGradlePlugin} *[\n\r]`, 'g'), (match) => {
                return `${match}${IMPORT_LINES.applyNewRelicGradle}\n`
              })

              FILE_HELPER
                .writeFile(file, fileBlob)
                .then((err) => {
                  if (err) {
                    LOG.debug(`Failed to add New Relic Gradle plugin to ${resolvedAppBuildGradlePath}`)
                    return err
                  } else {
                    LOG.info(`Updated Android Gradle with New Relic successfully`)
                  }
                })
            })
        })
      }
    })
    .catch((err) => {
      LOG.error(err)
    })
}
