const LOG = require('../../utils/logger');
const FILE_HELPER = require('../../utils/file-helper');
const CONSTANTS = require('../../constants');
const {IMPORT_LINES} = CONSTANTS;
const REACT_NATIVE_HELPERS = module.exports;

/**
 * @param filePath. The file to inject the New Relic import statement
 * @returns {Promise<any>}
 * Scans filePath and adds `import NewRelic from 'newrelic-mobile-rn'` to the list of imports. Fails if file not found
 * or import statement is already included in the file.
 */
REACT_NATIVE_HELPERS.injectNewRelicImport = (filePath) => {
  return new Promise(function (resolve, reject) {
    LOG.debug(`Scanning for ${filePath} to inject New Relic import`);
    FILE_HELPER
    .checkForFile(filePath)
    .then((files) => {
      if (files && files.length) {
        files.map((file) => {
          const touchedFile = `${process.cwd()}/${file}`;
          try {
            FILE_HELPER
            .readFileSync(touchedFile)
            .then((fileBlob) => {
              if (fileBlob.match(CONSTANTS.rnAgentPackageName)) {
                LOG.debug(`New Relic import already exists in ${touchedFile}`);
                return
              }

              const newFile = fileBlob.replace(/^([^]*)(import\s+[^;]*?;$)/m, (match) => {
                return `${match}\n${IMPORT_LINES.newRelicImport}`
              });

              FILE_HELPER.writeFile(touchedFile, newFile)
              .then((err) => {
                if (err) {
                  LOG.debug(`Failed to add New Relic import to ${touchedFile}`);
                  reject(err)
                }
                LOG.debug(`Added New Relic import to ${touchedFile}`);
                resolve(false)
              })
            })
          }
          catch (e) {
            reject(e)
          }
        })
      }
    })
    .catch((err) => {
      reject(err)
    })
  })
}

/**
 * @param filePath. The file pattern to inject the New Relic import statement
 * @returns {Promise<any>}
 * Scans filePath and removes `import NewRelic from 'newrelic-mobile-rn'` from the list of imports. Fails if file not found.
 */
REACT_NATIVE_HELPERS.removeNewRelicImport = (filePath) => {
  return new Promise(function (resolve, reject) {
    LOG.debug(`Scanning for ${filePath} to remove New Relic import`)
    FILE_HELPER
    .checkForFile(filePath)
    .then((files) => {
      files
      .map((file) => {
        const touchedFile = `${process.cwd()}/${file}`;

        FILE_HELPER
        .readFileSync(touchedFile)
        .then((fileBlob) => {
          let newFile = null
          const match = fileBlob.match(IMPORT_LINES.newRelicImport);

          if (match) {
            newFile = fileBlob.replace(IMPORT_LINES.newRelicImport, '')
          }

          if (newFile) {
            FILE_HELPER
            .writeFile(touchedFile, newFile)
            .then((err) => {
              if (err) {
                LOG.debug(`Failed to remove New Relic import to ${touchedFile}`)
                reject(err)
              }

              LOG.debug(`Removed New Relic import from ${touchedFile}`)
              resolve(false)
            })
          } else {
            resolve(false)
          }
        })
      })
    })
    .catch((err) => {
      reject(err)
    })
  })
}
