const reactNativeRemove = module.exports
const PUB_SUB = require('../../utils/pub-sub')
const LOG = require('../../utils/logger')
const CONSTANTS = require('../../constants')
const { IMPORT_LINES, XCODE_BUILD_PHASES, PATHS } = CONSTANTS;
const REACT_NATIVE_HELPERS = require('./react-native-helpers')
const FILE_HELPER = require('../../utils/file-helper')
const xcode = require('xcode')
const path = require('path');
const IOS = require('../../platforms/ios')


/**
 * Removes New Relic changes from a React Native app
 * @param args
 */
reactNativeRemove.handleReact = function (args) {
  LOG.info('Removing New Relic for React Native')

  if (args.platforms && args.platforms.length) {
    const platforms = args.platforms

    REACT_NATIVE_HELPERS.removeNewRelicImport(PATHS.rnApp)
    .then((err) => {
      if (!err) {
        LOG.info(`Removed New Relic import successfully`)
      }
    })
    .catch((err) => {
      LOG.debug(err)
    });

    platforms.map((platform) => {
      PUB_SUB.publish(`rn.remove.${platform.toLowerCase()}`, args)
    })
  }
}

/**
 * Removes the iOS-specific changes added to the React Native app during the `link` step.
 * @param args
 */
reactNativeRemove.handleIos = function (args = {}) {
  LOG.info('Removing New Relic from React Native iOS')
  const xCodeFilePath = args.xCodeFilePath || PATHS.xcodePbxproj
  FILE_HELPER
  .checkForFile(xCodeFilePath)
  .then((files) => {
    //hardcoded to the first file for now...
    const xcodeProject = xcode.project(files[0]);

    xcodeProject.parse((error) => {
      if (error) {
        LOG.debug(`Xcode loading error: ${error.code}`);
        LOG.error('Encountered an error loading the Xcode project file');
        return error
      }
      const iOSProject = new IOS(xcodeProject)

      iOSProject
        .removeBuildPhase(XCODE_BUILD_PHASES.postBuild)
        .removeBuildPhase(XCODE_BUILD_PHASES.uploadMaps)
        .removeBuildFiles(CONSTANTS.newRelicConfig)
        .removeCopyPhase(XCODE_BUILD_PHASES.copyNewRelicJson)
        .removeFileRefs(CONSTANTS.newRelicConfig)
        .removeFileInGroups(CONSTANTS.newRelicConfig)
        .finish()

      iOSProject
      .removeNewRelicResourcesBuildPhase()
      .finish()

      const modifiedProject = iOSProject.getXcodeProject().writeSync();

      FILE_HELPER.writeFile(files[0], modifiedProject)
      .then((err) => {
        if (err) {
          LOG.debug(`Failed to update: ${files[0]}`)
        }

        LOG.info(`Updated Xcode successfully`)
      })

    })

  })
  .catch((e) => {
    LOG.debug(e)
  })
}

/**
 * Removes the Android-specific changes added to the React Native app.
 * @param args
 */
reactNativeRemove.handleAndroid = function (args) {
  LOG.debug('Checking React Native Android project')
  const androidGradlePath = args.androidAppBuild || PATHS.appBuildGradle
  FILE_HELPER
  .checkForFile(androidGradlePath)
  .then((files) => {
    files.map((file) => {
      FILE_HELPER
      .readFileSync(file)
      .then((fileBlob) => {
        if(!fileBlob.match(IMPORT_LINES.applyNewRelicGradle)) {
          LOG.info("Nothing to remove from React Native Android project")
          return
        }

        LOG.info('Removing New Relic Gradle plugin import from React Native Android project')
        fileBlob = fileBlob.replace(`${IMPORT_LINES.applyNewRelicGradle}\n`, "")
        FILE_HELPER
        .writeFile(file, fileBlob)
        .then((err) => {
          if (err) {
            LOG.debug(`Failed to remove New Relic Gradle plugin from ${androidGradlePath}`)
            return err
          } else {
            LOG.info(`Removed Android Gradle with New Relic successfully.`)
          }
        })
      })
    })
  })
  .catch((e) => {
    LOG.error(e)
  })
}
