const rnUpload = module.exports;
const LOG = require('../../utils/logger');
const sourcemapUploader = require('../../services/sourcemap-uploader');
const FILE_HELPER = require('../../utils/file-helper');
const CONSTANTS = require('../../constants')

function shouldUpload(config, args) {
  if (config && config.licenseKey) {
    if (config.reactNative && config.reactNative.alwaysUploadJSSourceMap) {
      return true
    }
    return (args.buildType && args.buildType.toLowerCase().indexOf('release') !== -1)
  }

  LOG.debug(`No license key found. Cancelling upload.`);
  return false
}

/**
 * Uploads a JS source map file for symbolication of JS errors.
 * Will fail if cannot determine the version of the app from the package.json file or
 * if the newrelic.json file is missing app license information required for uploading source maps.
 */
rnUpload.upload = function (args) {
  LOG.verbose = args.verbose;

  if (!args.jsAppVersion) {
    try {
      args.jsAppVersion = require(`${process.cwd()}/package.json`).version
    }
    catch (e) {
      LOG.error('package.json version not found. aborting...');
      LOG.debug(`Failed to load package.json for upload: ${e}`);
      return
    }
  }

  LOG.debug(`JS App Version = ${args.jsAppVersion}`)
  LOG.debug(`Platform = ${args.platform}`)
  LOG.debug(`Build Type = ${args.buildType}`)
  LOG.debug(`File = ${args.file}`)
  LOG.debug(`Staging = ${args.staging}`)

  FILE_HELPER
  .checkForFile(CONSTANTS.newRelicConfig)
  .then((files) => {
    const config = require(`${process.cwd()}/${files[0]}`);
    const token = args.token || config.licenseKey;
    LOG.debug(`License Key = ${token}`)
    if (shouldUpload(config, args)) {
      sourcemapUploader.uploadMapFile({
        file: args.file,
        token: token,
        staging: args.staging,
        platform: args.platform,
        jsAppVersion: args.jsAppVersion,
      })
    } else {
      LOG.error(`Something went wrong uploading source map`);
      LOG.debug(`Config = ${config}`);
    }
  })
  .catch((err) => {
    LOG.error(`Upload error: ${err}`);
    LOG.info('No newrelic.json found');
  })
};
