const LOG = require('../../utils/logger')
const CONSTANTS = require('../../constants');
const { XCODE_BUILD_PHASES } = CONSTANTS
var util = require('util'),
  f = util.format

module.exports = class IOS {
  constructor(xcode = null) {
    this.name = 'ios'
    this.xcodeProject = xcode
  }

  getXcodeProject() {
    if (this.xcodeProject) {
      return this.xcodeProject
    }
    throw "no Xcode project was set."
  }

  // we can use our name as a guard against adding it multiple times
  buildPhaseGuard(newScriptName) {
    for (const script of this.buildScriptsCopy) {
      if (script && script.name && script.name.match(newScriptName)) {
        return true
      }
    }
    return false
  }

  addFileToBuildPhase(file, phase, group) {
    const localFile = Object.assign({}, file)
    localFile.uuid = this.getXcodeProject().generateUuid()
    localFile.group = group

    const newFile = {}
    newFile.comment = `${localFile.basename} in ${localFile.group}`
    newFile.value = localFile.uuid
    if(phase.files.filter(value => value.comment == newFile.comment).length === 0) {
      this.getXcodeProject().addToPbxBuildFileSection(localFile);
      phase.files.push(newFile)
    }
    return this
  }

  getBuildScripts(key) {
    return Object.keys(this.getXcodeProject().hash.project.objects[key])
      .map((buildScriptKey) => {
        if (this.getXcodeProject().hash.project.objects[key].hasOwnProperty(buildScriptKey)) {
          if (typeof this.getXcodeProject().hash.project.objects[key][buildScriptKey] == 'object') {
            return this.getXcodeProject().hash.project.objects[key][buildScriptKey]
          }
        }
      }).filter((el) => {
        return el != undefined;
      })
  }

  prepare(buildScriptsCopy) {
    this.buildScriptsCopy = buildScriptsCopy
    return this
  }

  addResourceFile(path, opt, group) {
    LOG.info(`Adding resource file: ${path}`)
    this.getXcodeProject().addResourceFile(path, opt, group)
    return this
  }

  addShellScriptBuildPhase(buildPhaseName, script, whichSh='/bin/sh') {
    if (this.buildPhaseGuard(buildPhaseName)) {
      LOG.debug(`\`${buildPhaseName}\` is already a build phase of the Xcode project`)
      return this
    }

    LOG.info(`Adding Xcode build phase: ${buildPhaseName}`)
    this.getXcodeProject().addBuildPhase([], 'PBXShellScriptBuildPhase', buildPhaseName, null, {shellPath: whichSh, shellScript: script})
    return this
  }

  removeFrameworksBuildPhase(name) {
    const phaseName = 'PBXFrameworksBuildPhase'
    Object
      .keys(this.getXcodeProject().hash.project.objects[phaseName])
      .filter(key => key.indexOf('comment') === -1)
      .map( key => {
          const frameworksPhase = this.getXcodeProject().hash.project.objects[phaseName][key]
          frameworksPhase.files
            .map((obj,index) => {
              if (obj.comment.indexOf(name) !== -1) {
                this.getXcodeProject().hash.project.objects[phaseName][key].files.splice(index,1)
              }
            })
      })
    return this.removeBuildFiles(name)
      .removePBXGroup(name)
  }

  removeBuildFiles(filename) {
    const phaseName = 'PBXBuildFile'
    Object
      .keys(this.getXcodeProject().hash.project.objects[phaseName])
      .map((key) => {
        const obj = this.getXcodeProject().hash.project.objects[phaseName][key]
        if (key.indexOf('comment') === -1) {
          if (obj && obj.fileRef_comment && obj.fileRef_comment.indexOf(filename) !== -1) {
            delete this.getXcodeProject().hash.project.objects[phaseName][key]
          }
        } else {
          if (obj.indexOf(filename)!==-1) {
            delete this.getXcodeProject().hash.project.objects[phaseName][key]
          }
        }
      })
    return this.removeFileRefs(filename)
  }

  removeBuildPhaseReferences(name) {
    const phaseName = 'PBXNativeTarget'
    Object.keys(this.getXcodeProject().hash.project.objects[phaseName])
      .filter(obj => obj.indexOf('comment') === -1 )
      .map( key => {
        const target = this.getXcodeProject().hash.project.objects[phaseName][key]
        if (target) {
           target.buildPhases.map((item, index)=>{
             if (item.comment.indexOf(name) !== -1) {
               this.getXcodeProject().hash.project.objects[phaseName][key].buildPhases.splice(index,1)
             }
           })
        }
      })
    return this
  }

  removeFileInGroups(fileName) {
    let groups = this.getXcodeProject().hash.project.objects['PBXGroup']
    let groupsWithNewRelic = Object.keys(groups)
      .filter(i => i.indexOf('comment') === -1)
      .filter( i => groups[i].children.filter( obj => obj.comment === CONSTANTS.newRelicConfig).length > 0)

    groupsWithNewRelic.forEach(value => {
      let newRelicJsonIndex = groups[value].children.findIndex( obj => obj.comment === CONSTANTS.newRelicConfig)
      this.getXcodeProject().hash.project.objects['PBXGroup'][value].children.splice(newRelicJsonIndex,1)
    })

    return this
  }

  removeFileRefs(filename) {
    const phaseName = 'PBXFileReference'
      Object
      .keys(this.getXcodeProject().hash.project.objects[phaseName])
      .map((key) => {
        const obj = this.getXcodeProject().hash.project.objects[phaseName][key]
        if (key.indexOf('comment') === -1) {
          if (obj && obj.name && obj.name.indexOf(filename) !== -1) {
            delete this.getXcodeProject().hash.project.objects[phaseName][key]
          }
        } else {
          if (obj.indexOf(filename)!==-1) {
            delete this.getXcodeProject().hash.project.objects[phaseName][key]
          }

        }
      })
    return this

  }

  removePhase(comment, phaseName) {
    if(!this.getXcodeProject().hash.project.objects[phaseName]) {
      return this
    }

    Object
      .keys(this.getXcodeProject().hash.project.objects[phaseName])
      .map((buildScriptKey, index) => {
        if (this.getXcodeProject().hash.project.objects[phaseName].hasOwnProperty(buildScriptKey)) {
          if (typeof this.getXcodeProject().hash.project.objects[phaseName][buildScriptKey] == 'object') {
            const project = this.getXcodeProject().hash.project.objects[phaseName][buildScriptKey]
            if (project && project.name && project.name.match(comment)) {
              delete this.getXcodeProject().hash.project.objects[phaseName][buildScriptKey]
            }
          } else if (typeof this.getXcodeProject().hash.project.objects[phaseName][buildScriptKey] == 'string') {
            const string = this.getXcodeProject().hash.project.objects[phaseName][buildScriptKey]
            if (string && string.match(comment)) {
              delete this.getXcodeProject().hash.project.objects[phaseName][buildScriptKey]
            }
          }
        }
      })
    return this
  }

  removeCopyPhase(comment) {
    const phaseType = 'PBXCopyFilesBuildPhase'
    return this.removePhase(comment, phaseType).removeBuildPhaseReferences(comment)
  }

  removeBuildPhase(comment) {
    const phaseName = 'PBXShellScriptBuildPhase'
    return this.removePhase(comment, phaseName).removeBuildPhaseReferences(comment)
  }

  /* Add newrelic.json to build copy phase
    *  Some steps need to be followed to add a file cleaning to the xcode project
    *  1. add a PBXFileRef - this tells the xcode project where to find the file
    *  2. add the PBXFileRef to a PBXGroup - tells the project how to display this file. otherwise it will be added to a 'lost ref' folder
    *  3. add a PBXBuildFile - this will reference the PBXFileRef
    *  4. add a PBXCopyFileBundlePhase - this is the phase that copies the files into the application artifact, references the PBXBuildFile.
    * */
  addCopyNewRelicConfigBuildPhase(buildPhaseName, configFilePath) {
    if (this.copyConfigBuildPhaseExists()) {
      LOG.debug(`\`${buildPhaseName}\` already a build phase of the Xcode project`)
      return this
    }

    LOG.info(`Adding New Relic build phase: ${buildPhaseName}`);

    // find the root group
    let groups = this.getXcodeProject().hash.project.objects['PBXGroup']
    let rootGroup = Object.keys(groups)
      .filter(i => i.indexOf('comment') === -1) //no need for comment keys
      .find(i =>
        groups[i].indentWidth !== undefined
      ) // the root group has no name

    // add file
    let file = this.getXcodeProject().addFile(configFilePath, rootGroup)
    file.uuid = this.xcodeProject.generateUuid()

    // create PBXBuildFile
    this.getXcodeProject().addToPbxBuildFileSection(file)

    // add PBXCopyFilesBuildPhases if it doesn't exist
    let buildPhaseType = "PBXCopyFilesBuildPhase"
    if (!this.getXcodeProject().hash.project.objects[buildPhaseType]) {
      this.getXcodeProject().hash.project.objects[buildPhaseType] = new Object();
    }

    // add copy phase to PBXCopyFilesBuildPhases
    // create build phase
    let buildPhaseUuid = this.getXcodeProject().generateUuid()
    let buildPhaseCommentKey = f("%s_comment", buildPhaseUuid)
    let buildPhaseTargetUuid = this.getXcodeProject().getFirstTarget().uuid
    let buildPhase = {
      isa: buildPhaseType,
      name: '"' + XCODE_BUILD_PHASES.copyNewRelicJson + '"',
      buildActionMask: 2147483647,
      dstSubfolderSpec: 7,
      dstPath: '""',
      files: [file.uuid],
      runOnlyForDeploymentPostprocessing: 0
    }
    if (!this.getXcodeProject().hash.project.objects[buildPhaseType][buildPhaseUuid]) {
      this.getXcodeProject().hash.project.objects[buildPhaseType][buildPhaseUuid] = buildPhase;
      this.getXcodeProject().hash.project.objects[buildPhaseType][buildPhaseCommentKey] = buildPhaseName;
    }

    // add copy phase to the main target
    if (this.getXcodeProject().hash.project.objects['PBXNativeTarget'][buildPhaseTargetUuid]['buildPhases']) {
        this.getXcodeProject().hash.project.objects['PBXNativeTarget'][buildPhaseTargetUuid]['buildPhases'].push({
          value: buildPhaseUuid,
          comment: buildPhaseName
        })
    }
    return this
  }

  copyConfigBuildPhaseExists() {
    const copyFilesBuildSection = this.getXcodeProject().hash.project.objects['PBXCopyFilesBuildPhase']
    if (copyFilesBuildSection) {
      for (let phase in copyFilesBuildSection){
        if(copyFilesBuildSection[phase] === XCODE_BUILD_PHASES.copyNewRelicJson) {
          return true
        }
      }
    }
    return false
  }

  removePBXGroup(file) {
    const phaseName = 'PBXGroup'
    Object
      .keys(this.getXcodeProject().hash.project.objects[phaseName])
      .filter(key => key.indexOf('comment') === -1)
      .map(key => {
        this.getXcodeProject().hash.project.objects[phaseName][key].children
          .map((object,index) => {
              if (object.comment.indexOf(file) !== -1) {
                this.getXcodeProject().hash.project.objects[phaseName][key].children.splice(index,1)
              }
          })
      })
    return this
  }

  removeNewRelicResourcesBuildPhase() {
    const key = 'PBXResourcesBuildPhase'
    Object
      .keys(this.getXcodeProject().hash.project.objects[key])
      .map((buildScriptKey, index) => {
        if (this.getXcodeProject().hash.project.objects[key].hasOwnProperty(buildScriptKey)) {
          if (typeof this.getXcodeProject().hash.project.objects[key][buildScriptKey] == 'object') {
            const project = this.getXcodeProject().hash.project.objects[key][buildScriptKey]
            for (const file of project.files) {
              if (file.comment.indexOf(CONSTANTS.newRelicConfig) > -1) {
                delete this.getXcodeProject().hash.project.objects[key][buildScriptKey]
              }
            }
          }
        }
      })
    return this
  }

  finish() {
    this.buildScriptsCopy = null
    return this
  }

}
